/* "eqcs_qc.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_QC_H_
#ifndef __GNUG__
#pragma interface
#endif
#define	_EQCS_QC_H_

#include "eqcs_gate_array.h"

class EqcsState;
class EqcsQcRep;

/*****************************************************************************\
*	quantum computer														  *
\*****************************************************************************/

class EqcsQc: public EqcsHandle {
public:
	typedef EqcsHandle base_t;
	typedef EqcsQcRep rep_t;

	EqcsQc(rep_t *rep=0);
	EqcsQc(const EqcsState &state);

	// Prepare the quantum computer in some state.
	void prepare(const EqcsState &state) const;
	// Observe some bits.
	EqcsState observe(unsigned long bits) const;
	// Perform one instruction.
	void perform(const EqcsGate &gate) const;
	// Perform a sequence of instructions.
	void perform(const EqcsGateArray &array) const;

    EqcsBitset measure(unsigned long bits) const;
	EqcsState state() const;

	// Some technical uitilities.
	inline rep_t *operator->() const
		{ return (rep_t *)EqcsHandle::rep(); }
	inline rep_t *rep() const
		{ return (rep_t *)EqcsHandle::rep(); }
};

#endif	//	_EQCS_QC_H_
