/* "eqcs_state.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifdef __GNUG__
#pragma implementation
#endif

#include "eqcs_state_rep.h"
#include "eqcs_state.h"

/*****************************************************************************\
*	state																	  *
\*****************************************************************************/

EqcsState::EqcsState(rep_t *rep) : base_t(rep)
	{}

EqcsState::EqcsState(unsigned long bits) : base_t(new rep_t(bits))
	{}

bool EqcsState::operator<(const EqcsState &state) const
	{ return rep()->less_then(state.rep()); }

EqcsState EqcsState::operator-() const
	{ return rep()->neg(); }

EqcsState EqcsState::operator+=(const EqcsState &state) const
	{ return rep()->add(state.rep()); }

EqcsState EqcsState::operator-=(const EqcsState &state) const
	{ return rep()->sub(state.rep()); }

EqcsState EqcsState::operator*=(const EqcsState::complex_t &c) const
	{ return rep()->mul(c); }

EqcsState EqcsState::collapse(unsigned long bits) const
	{ return rep()->collapse(bits); }

EqcsBitset EqcsState::measure(unsigned long bits) const
	{ return rep()->measure(bits); }

ostream &EqcsState::put_bits(ostream &os, int nregs, int regwidth) const
	{ return rep()->put_bits(os, nregs, regwidth); }

ostream &EqcsState::put_nums(ostream &os, int nregs, int regwidth) const
	{ return rep()->put_nums(os, nregs, regwidth); }

EqcsState operator+(const EqcsState &l, const EqcsState &r)
	{ return l->clone()->add(r.rep()); }

EqcsState operator-(const EqcsState &l, const EqcsState &r)
	{ return l->clone()->sub(r.rep()); }

EqcsState operator*(const EqcsState::complex_t &l, const EqcsState &r)
	{ return r->clone()->mul(l); }

EqcsState operator*(const EqcsState &l, const EqcsState::complex_t &r)
	{ return l->clone()->mul(r); }

ostream &operator<<(ostream &os, const EqcsState &state)
	{ return state->put(os); }

EqcsState::omanip2_t bits(const EqcsState &state, int n, int width)
	{ return EqcsState::omanip2_t(state, &EqcsState::put_bits, n, width); }

EqcsState::omanip2_t nums(const EqcsState &state, int n, int width)
	{ return EqcsState::omanip2_t(state, &EqcsState::put_nums, n, width); }
