/* "eqcs_state_rep.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_STATE_REP_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_STATE_REP_H_

#include <iostream>
#include <complex>
#include <map>
#include "eqcs_body.h"
#include "eqcs_bitset.h"

using namespace std;
class EqcsMatrixRep;
class EqcsLambdaRep;
class EqcsGateRep;

/*****************************************************************************\
*	state representation													  *
\*****************************************************************************/

class EqcsStateRep: public EqcsBody {
public:
	friend class EqcsMatrixRep;
	friend class EqcsLambdaRep;

	typedef EqcsBitset bitset_t;
	typedef complex<double> complex_t;
	typedef map<bitset_t, complex_t> terms_t;
	typedef terms_t::iterator iterator;
	typedef terms_t::const_iterator const_iterator;

	EqcsStateRep(int nqubits, int);
	EqcsStateRep(unsigned long bits);
	EqcsStateRep(int nqubits, const terms_t &terms);
	virtual ~EqcsStateRep();

	inline EqcsStateRep *clone() const
		{ return new EqcsStateRep(_nqubits, _terms); }

	// Get the terms.
	inline terms_t &terms()
		{ return _terms; }
	inline const terms_t &terms() const
		{ return _terms; }

	// Define a partial order.
	bool less_then(const EqcsStateRep *rep) const;

	// Negate the state.
	EqcsStateRep *neg() const;

	// Add a state.
	EqcsStateRep *add(const EqcsStateRep *rep);
	EqcsStateRep *sub(const EqcsStateRep *rep);
	// Multiply by a complex number.
	EqcsStateRep *mul(const complex_t &c);

    bitset_t measure(const bitset_t &bitest) const;

    inline bitset_t measure(unsigned long bits) const
	  { return measure(bitset_t::bits2bitset(bits)); }

	EqcsStateRep *collapse(unsigned long bits) const;

	// Calculate a state's norm, i.e. |q0|^2 + |q1|^2 + ...
	friend double norm(const EqcsStateRep *state);

	// Get the number of qubits.
	inline int nqubits() const
		{ return _nqubits; }
	// Get the Hilbert space's dimension.
	inline unsigned long dim() const
		{ return 1 << _nqubits; }

	// Write a state to a stream.
	ostream &put(ostream &os) const;
	ostream &put_bits(ostream &os, int nregs, int regwidth) const;
	ostream &put_nums(ostream &os, int nregs, int regwidth) const;
private:
	int			_nqubits;	// The number of qubits.
	terms_t		_terms;		// The complex amplitudes.
};

#endif	//	_EQCS_STATE_REP_H_
